import numpy as np
import math
import pandas as pd
import matplotlib as mpl
import matplotlib.pyplot as plt
import seaborn as sns
import sys
import smfishHmrf.reader as reader
import scipy
import scipy.stats
from sklearn.decomposition import PCA
from sklearn.utils.extmath import randomized_svd
from sklearn.manifold import TSNE
#sys.path.insert(0, "/home/qz64/hmrf.submit")
import os
#import rank_transform
#import calculate_overlap
sys.setrecursionlimit(10000)

#gene by gene correction
#requires expr, Xcen, field, FDs, num_bin
def calc_bias_moving(expr=None, centroid=None, field=None, interest_field=None, num_bin=10):
	if expr is None or centroid is None or field is None or interest_field is None:
		sys.stderr.write("Need expr, centroid, field, interest_field\n")
		return ;
	Xcen = centroid
	FDs = interest_field
	good_i = []
	for i in range(Xcen.shape[0]):
		if field[i] in set(FDs):
			good_i.append(i)
	good_i = np.array(good_i)
	expr = expr[good_i]
	Xcen = Xcen[good_i]
	field = field[good_i]
	#field specific average
	average = {}
	isCenter = np.empty(Xcen.shape[0], dtype="int32")
	for i in range(Xcen.shape[0]):
		if abs(Xcen[i,0])<1.0 and abs(Xcen[i,1])<1.0:
			isCenter[i] = 1
		else:
			isCenter[i] = 0
	for fd in sorted(FDs):
		m = np.where((field==fd) & (isCenter==1))[0]
		if np.isnan(np.mean(expr[m])):
			#if roi=="whole.brain.1e" and fd==39:
			#	average[fd] = average[38]
			print "Warning no cells in the center for field", fd, average[38]
		else:
			average[fd] = np.mean(expr[m])

	bin_size = 4.0 / num_bin
	cls = {}
	Xcen2 = Xcen
	bins = np.empty((num_bin, num_bin), dtype="float32")
	ct = np.empty((num_bin, num_bin), dtype="int32")
	val = {}
	for x in range(num_bin):
		for y in range(num_bin):
			bins[x,y] = 0
			ct[x,y] = 0
			val.setdefault((x,y), [])
	for i in range(Xcen2.shape[0]):
		t1 = int((max(min(Xcen2[i,0], 1.99), -1.99) - (-2.0)) / bin_size) 
		t2 = int((max(min(Xcen2[i,1], 1.99), -1.99) - (-2.0)) / bin_size) 
		val[(t1,t2)].append((expr[i], average[field[i]]))
		#print goi, t1, t2, expr[i], average[field[i]]
	moving_bins = [-2, -1, 0]
	#moving_bins = [-2, -1, 0, 1, 2]
	for x in range(num_bin):
		for y in range(num_bin):
			all_val = []
			for a in moving_bins:
				for b in moving_bins:
					if x+a>=0 and y+b>=0 and x+a<num_bin and y+b<num_bin: 
						all_val = all_val + val[(x+a, y+b)]
			all_expr = [t[0] for t in all_val]
			all_fd = [t[1] for t in all_val]
			bins[x,y] = np.sum(np.array(all_expr)) - np.sum(np.array(all_fd))
			#print goi, x,y,bins[x,y]
			ct[x,y] = len(all_expr)
			if ct[x,y]==0:
				bins[x,y] = 0
			else:
				bins[x,y] = bins[x,y] / float(ct[x,y])
	bins3 = np.empty((num_bin, num_bin), dtype="float32")
	for x in range(num_bin):
		for y in range(num_bin):
			bins3[x,y] = 0
	#xn = np.empty((num_bin, num_bin), dtype=np.uint8)
	for x in range(num_bin):
		for y in range(num_bin):
			if bins[y,num_bin-x-1]==0:
				#xn[x,y] = 127
				bins3[x,y] = 0
			else:
				#xn[x,y] = max(0, min(int((bins[y,num_bin-x-1] + 5.0) * 25.5),255))
				bins3[x,y] = bins[y,num_bin-x-1]
				#if xn[x,y] == 127:
				#	xn[x,y] = 128
				#	bins3[x,y] = 0
	#np.ndarray.flatten(xn).tofile("/tmp/example.%s.raw" % "tmp")
	#print np.max(bins3), np.min(bins3)
	return bins3

#bias is like the field bias map (across field averages)
def do_pca(expr=None, expr_bias=None, centroid=None, field=None, interest_field=None, num_bin=50, top_comp_remove=5):
	if expr is None or expr_bias is None or centroid is None or field is None or interest_field is None:
		sys.stderr.write("Need expr, expr_bias, centroid, field, interest_field\n")
		return ;
	Xcen = centroid
	FDs = interest_field
	ngene = expr.shape[0]
	U,Sigma,VT = randomized_svd(np.transpose(expr_bias), n_components=ngene, n_iter=20, random_state=None)
	print Sigma
	#print U.shape #2500, 125
	#print VT.shape #125, 125
	for i in range(top_comp_remove, Sigma.shape[0]):
		Sigma[i] = 0
	U_Sigma = np.matmul(U, np.diag(Sigma))
	transformed = np.matmul(U_Sigma, VT)
	expr_bias = np.transpose(transformed)

	bin_size = 4.0 / num_bin
	for ind in range(expr.shape[0]):
		bias = np.reshape(expr_bias[ind,], (num_bin, num_bin)) #reshaped bins
		for i in range(Xcen.shape[0]):
			if not field[i] in set(FDs):
				continue
			t1 = int((max(min(Xcen[i,0], 1.99), -1.99) - (-2.0)) / bin_size) 
			t2 = int((max(min(Xcen[i,1], 1.99), -1.99) - (-2.0)) / bin_size) 		
			x1 = num_bin-t2-1
			y1 = t1
			expr[ind,i] = expr[ind,i] - bias[x1,y1]
	return expr

def get_pivot2(n):
	df = {}
	nd1 = n.shape[0]
	nd2 = n.shape[1]
	for i in range(1, nd1+1):
		for j in range(1, nd2+1):
			df.setdefault(i, {})
			df[i][j] = n[i-1,j-1]
	df2 = pd.DataFrame(df)
	return df2

def plot_pca(expr_bias=None, num_bin=None, top_pc=None, out_file=None):
	if expr_bias is None or num_bin is None or top_pc is None or out_file is None:
		sys.stderr.write("Need expr_bias, num_bin, top_pc, out_file\n")
		return ;
	ngene = expr.shape[0]
	U,Sigma,VT = randomized_svd(np.transpose(expr_bias), n_components=ngene, n_iter=20, random_state=None)
	nrow = int(top_pc / 4)
	ncol = 4
	if top_pc%4!=0:
		nrow+=1
	unit_x = 3.5
	unit_y = 3
	f2, axn = plt.subplots(nrow, ncol, figsize=(unit_x*ncol, unit_y*nrow))
	for p_id in range(top_pc):
		b_bias = np.reshape(U[:, p_id], (num_bin, num_bin))
		sns.heatmap(get_pivot2(np.transpose(b_bias)), annot=False, fmt=".2f", ax=axn.flat[p_id])
	f2.savefig(out_file)

if __name__=="__main__":
	directory = "hmrfem.whole.brain.e.fd0-48.correlated.gene"
	hippo_FDs = [30,31,32,33,34,35,36,37,38,39,40,41,42]
	cortex_FDs = [0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19]
	temporal_cortex_FDs = [20,21,22,23,24,25,26,27,28,29]
	roi = "whole.brain.e"

	#FDs = hippo_FDs + cortex_FDs + temporal_cortex_FDs
	FDs = hippo_FDs + cortex_FDs
	workdir = "/home/qz64/hmrf.submit"
	genes = reader.read_genes("%s/%s/genes" % (workdir, directory))
	Xcen, field = reader.read_coord("%s/%s/f%s.coordinates.2.txt" % (workdir, directory, roi)) #note the stitched version use the .2 version

	expr = np.empty((len(genes), Xcen.shape[0]), dtype="float32")
	for ind,g in enumerate(genes):
		expr[ind,:] = reader.read_expression("%s/%s/f%s.gene.%s.txt" % (workdir, directory, roi, g))
	
	expr_bias = np.empty((len(genes), 50*50), dtype="float32")
	for ind,g in enumerate(genes):
		bins = calc_bias_moving(expr=expr[ind,:], centroid=Xcen, field=field, interest_field=FDs, num_bin=50)
		expr_bias[ind,:] = np.ndarray.flatten(bins)

	corrected_expr = do_pca(expr=expr, expr_bias=expr_bias, centroid=Xcen, field=field, interest_field=FDs, num_bin=50, top_comp_remove=5)

	plot_pca(expr_bias=expr_bias, num_bin=50, top_pc=5, out_file="bias_component.pdf")

		
	if not os.path.isdir("%s/%s/bias_corrected" % (workdir,directory)):
		os.mkdir("%s/%s/bias_corrected" % (workdir, directory))
	
	for ind,g in enumerate(genes):
		print "Doing", g
		reader.write_expression(corrected_expr[ind,:], "%s/%s/bias_corrected/f%s.gene.%s.txt" % (workdir, directory, roi, g))
	



import matplotlib as mpl
import matplotlib.pyplot as plt
import scipy
import scipy.stats
import sys
import re
import os
import numpy as np
import math
import pandas as pd
import seaborn as sns
from scipy.spatial.distance import squareform, pdist
from scipy.stats import percentileofscore
import smfishHmrf.reader as reader
from smfishHmrf.HMRFInstance import HMRFInstance	
from smfishHmrf.DatasetMatrix import DatasetMatrix, DatasetMatrixSingleField, DatasetMatrixMultiField

def gene_expression(this_dataset, goi=None, foi=None, vmax=2.0, vmin=-2.0, \
edgecolors="none", title=True, colormap="Reds", size_factor=2, \
ncol=10, dot_size=20, outfile=None, xlim=(0,100), ylim=(0,100)):

	assert isinstance(this_dataset, DatasetMatrix), \
	"this_dataset must be of DatasetMatrixMultiField or DatasetMatrixSingleField"

	multifield = False
	if isinstance(this_dataset, DatasetMatrixMultiField):
		multiField = True
	
	multigene = False
	if isinstance(goi, list) and len(goi)>1:
		multigene = True	
	if len(goi)==1:
		goi = goi[0]

	assert multifield==False or multifield==False, "multifield, multigene is not supported"

	if multifield==True and multigene==False: #single gene, multifield
		gene_expression_multi_field(this_dataset, goi=goi, foi=foi, vmax=vmax, vmin=vmin, \
		edgecolors=edgecolors, title=title, colormap=colormap, size_factor=size_factor, \
		ncol=ncol, dot_size=dot_size, outfile=outfile)
	elif multifield==False and multigene==False: #single gene, single field
		gene_expression_single_field(this_dataset, goi=goi, vmax=vmax, vmin=vmin, \
		edgecolors=edgecolors, title=title, colormap=colormap, size_factor=size_factor, \
		dot_size=dot_size, outfile=outfile)
	elif multifield==False and multigene==True: #multigene single field
		multigene_expression_single_field(this_dataset, goi=goi, vmax=vmax, vmin=vmin, \
		edgecolors=edgecolors, title=title, colormap=colormap, ncol=ncol, size_factor=size_factor, \
		dot_size=dot_size, outfile=outfile, xlim=xlim, ylim=ylim)

	
#foi is field of interest
def gene_expression_multi_field(this_dataset, goi, foi=None, vmax=2.0, vmin=-2.0, \
edgecolors="none", title=True, colormap="Reds", size_factor=2, \
ncol=10, dot_size=20, outfile=None):
	assert isinstance(foi, (None, list)), "foi must be a list or None"
	assert isinstance(goi, basestring), "goi must be a basestring"
	assert isinstance(this_dataset, DatasetMatrixMultiField), "Dataset must be DatasetMatrixMultiField"
	X = this_dataset.expr

	Xcen = this_dataset.Xcen
	genes = this_dataset.genes
	gene_map = {}
	for ig,g in enumerate(genes):
		gene_map[g] = ig
	field = this_dataset.field

	FD = None
	if foi is None:
		FD = np.unique(field).tolist()
	else:
		FD = foi

	Xcen2 = np.copy(Xcen)
	nrow = len(FD) / ncol
	if len(FD)%ncol>0:
		nrow+=1

	f, axn = plt.subplots(nrow, ncol, figsize=(ncol * size_factor, nrow * size_factor))
	plt.subplots_adjust(hspace=0.01, wspace=0.01)
	cm = plt.cm.get_cmap(colormap)

	ct = 0
	gid = gene_map[goi]
	for fd in FD:
		m = np.where(field==fd)[0]
		axn.flat[ct].scatter(Xcen2[m,0], Xcen2[m,1], s=dot_size, c=X[gid,m], edgecolors=edgecolors, \
		cmap=cm, vmin=vmin, vmax=vmax)
		#axn.flat[ct].set_xlim(5, 30)
		#axn.flat[ct].set_ylim(5, 30)
		axn.flat[ct].title.set_visible(False)
		axn.flat[ct].set_axis_bgcolor("white")
		#plt.axis("off")
		if title:
			#axn.flat[ct].text(5, 25, "%s" % g)
			axn.flat[ct].annotate("%s (FD=%d)" % (goi, fd), (0.5, 0.95), xycoords='figure fraction', ha='center')
		ct+=1

	if outfile is None:
		plt.show()
	else:
		f.savefig(outfile)

def gene_expression_single_field(this_dataset, goi, vmax=2.0, vmin=-2.0, \
edgecolors="none", title=True, colormap="Reds", size_factor=2, \
dot_size=20, outfile=None):
	assert isinstance(goi, basestring), "gene must be basestring"
	assert isinstance(this_dataset, DatasetMatrixSingleField), "Dataset must be DatasetMatrixSingleField"
	X = this_dataset.expr
	Xcen = this_dataset.Xcen
	genes = this_dataset.genes
	gene_map = {}
	for ig,g in enumerate(genes):
		gene_map[g] = ig

	Xcen2 = np.copy(Xcen)

	f, axn = plt.subplots(1, 1, figsize=(size_factor, size_factor))

	plt.subplots_adjust(hspace=0.01, wspace=0.01)
	cm = plt.cm.get_cmap(colormap)

	gid = gene_map[goi]
	axn.scatter(Xcen2[:,0], Xcen2[:,1], s=dot_size, c=X[gid,:], edgecolors=edgecolors, \
	cmap=cm, vmin=vmin, vmax=vmax)
	axn.title.set_visible(False)
	axn.set_axis_bgcolor("white")
	if title:
		axn.annotate("Gene %s" % goi, (0.5, 0.95), xycoords='figure fraction', ha='center')
	if outfile is None:
		plt.show()
	else:
		f.savefig(outfile)

def multigene_expression_single_field(this_dataset, goi, vmax=2.0, vmin=-2.0, \
edgecolors="none", title=True, colormap="Reds", ncol=10, size_factor=2, \
dot_size=20, outfile=None, xlim=(0,100), ylim=(0,100)):
	assert isinstance(goi, list), "goi must be a list"
	assert isinstance(this_dataset, DatasetMatrixSingleField), "Dataset must be DatasetMatrixSingleField"
	X = this_dataset.expr
	Xcen = this_dataset.Xcen
	genes = this_dataset.genes
	gene_map = {}
	for ig,g in enumerate(genes):
		gene_map[g] = ig
	Xcen2 = np.copy(Xcen)

	nrow = len(genes) / ncol
	if len(genes)%ncol>0:
		nrow+=1

	f, axn = plt.subplots(nrow, ncol, figsize=(ncol * size_factor, nrow * size_factor), \
	sharex=True, sharey=True)

	plt.subplots_adjust(hspace=0.01, wspace=0.01)
	cm = plt.cm.get_cmap(colormap)

	ct = 0
	for g in goi:
		gid = gene_map[g]
		axn.flat[ct].scatter(Xcen2[:,0], Xcen2[:,1], s=dot_size, c=X[gid,:], edgecolors=edgecolors, \
		cmap=cm, vmin=vmin, vmax=vmax)
		axn.flat[ct].set_xlim(xlim[0], xlim[1])
		axn.flat[ct].set_ylim(ylim[0], ylim[1])
		axn.flat[ct].title.set_visible(False)
		axn.flat[ct].set_axis_bgcolor("white")
		#plt.axis("off")
		if title:
			#axn.flat[ct].text(5, 25, "%s" % g)
			axn.flat[ct].annotate("Gene %s" % g, (0.5, 0.95), xycoords='figure fraction', ha='center')
		ct+=1

	if outfile is None:
		plt.show()
	else:
		f.savefig(outfile)


def domain(this_HMRF, this_k, this_beta, dot_size=45, foi=None, xlim=(0,100), \
ylim=(0,100), colormap="RdYlBu_r", size_factor=10, ncol=10, outfile=None):
	assert isinstance(this_HMRF, HMRFInstance), "this_HMRF is not of HMRFInstance"
	#assert isinstance(this_HMRF.dataset, DatasetMatrix), "the dataset in this_HMRF must be of DatasetMatrixSingleField"
	if isinstance(this_HMRF.dataset, DatasetMatrixSingleField):
		domain_single_field(this_HMRF, this_k, this_beta, dot_size=dot_size, \
		size_factor=size_factor, colormap=colormap, outfile=outfile)
	else:
		domain_multi_field(this_HMRF, this_k, this_beta, foi=foi, dot_size=dot_size, xlim=xlim,\
		ylim=ylim, colormap=colormap, size_factor=size_factor, ncol=ncol, outfile=outfile)


def domain_single_field(this_HMRF, this_k, this_beta, dot_size=45, \
size_factor=10, colormap="RdYlBu_r", outfile=None):
	assert isinstance(this_HMRF, HMRFInstance), "this_HMRF is not of HMRFInstance"
	assert isinstance(this_HMRF.dataset, DatasetMatrixSingleField), "the dataset in this_HMRF must be of DatasetMatrixSingleField"
	Xcen = this_HMRF.dataset.Xcen
	f, axn = plt.subplots(1, 1, figsize=(size_factor, size_factor))
	plt.subplots_adjust(hspace=0.01, wspace=0.01)
	cm = plt.cm.get_cmap(colormap)
	cls = this_HMRF.domain[(this_k, this_beta)]
	axn.scatter(Xcen[:,0], Xcen[:,1], s=dot_size, c=cls, edgecolors="gray", \
	cmap=cm, vmin=1, vmax=max(cls))
	#axn.set_xlim(xlim[0], xlim[1])
	#axn.set_ylim(ylim[0], ylim[1])
	if outfile is None:
		plt.show()
	else:
		f.savefig(outfile)

def domain_multi_field(this_HMRF, this_k, this_beta, foi=None, dot_size=45, xlim=(0,100), \
ylim=(0,100), colormap="RdYlBu_r", size_factor=10, ncol=10, outfile=None):
	assert isinstance(this_HMRF, HMRFInstance), "this_HMRF is not of HMRFInstance"
	assert isinstance(this_HMRF.dataset, DatasetMatrixMultiField), "the dataset in this_HMRF must be of DatasetMatrixMultiField"
	Xcen = this_HMRF.dataset.Xcen
	field = this_HMRF.dataset.field
	FD = None
	if foi is None:
		FD = np.unique(field).tolist()
	else:
		FD = foi

	nrow = len(FD) / ncol
	if len(FD)%ncol>0:
		nrow+=1

	f, axn = plt.subplots(nrow, ncol, figsize=(ncol * size_factor, nrow * size_factor), \
	sharex=True, sharey=True)
	plt.subplots_adjust(hspace=0.01, wspace=0.01)
	cm = plt.cm.get_cmap(colormap)
	cls = this_HMRF.domain[(this_k, this_beta)]

	ct = 0
	for fd in FD:
		m = np.where(field==fd)[0]
		axn.flat[ct].scatter(Xcen[m,0], Xcen[m,1], s=dot_size, c=cls[m], edgecolors=edgecolors, \
		cmap=cm, vmin=1, vmax=max(cls))
		axn.flat[ct].set_xlim(xlim[0], xlim[1])
		axn.flat[ct].set_ylim(ylim[0], ylim[1])
		axn.flat[ct].title.set_visible(False)
		axn.flat[ct].set_axis_bgcolor("white")
		if title:
			axn.flat[ct].annotate("FD=%d" % fd, (0.5, 0.95), xycoords='figure fraction', ha='center')
		ct+=1

	if outfile is None:
		plt.show()
	else:
		f.savefig(outfile)

def domain_expression(this_HMRF, this_k, this_beta, vmin=-2.0, vmax=2.0, \
colormap="RdYlBu_r", outfile=None):
#directory, roi, boi, Xcen, FDs, genes, k, field):
	assert isinstance(this_HMRF, HMRFInstance), "this_HMRF is not of HMRFInstance"
	expr = this_HMRF.dataset.expr
	cls = this_HMRF.domain[(this_k, this_beta)]
	genes = this_HMRF.dataset.genes
	gene_map = {}
	for ig,g in enumerate(genes):
		gene_map[g] = ig

	nj_union = []
	for ki in range(1, this_k+1):
		m = np.where(cls==ki)[0]
		nx = []
		for g in genes:
			es = expr[gene_map[g], m]
			nx.append((ki, g, np.mean(es), np.std(es)))
		nx.sort(lambda x,y:cmp(x[2], y[2]), reverse=True)
		for ni, nj, nk, nl in nx[:10]:
			nj_union = nj_union + [nj]
		for ni, nj, nk, nl in nx[-10:]:
			nj_union = nj_union + [nj]

	nj_union = set(nj_union)
	#nj_union = genes
	#nj_union = list(set(nj_union))
	cluster_expr = np.empty((len(nj_union), k), dtype="float32")
	nj_union_title = []
	for n in nj_union:
		nj_union_title.append(n.title())
	nt = {}
	for ki in range(1, this_k+1):
		m = np.where(cls==ki)[0]
		nx = []
		for g in nj_union:
			es = expr[gene_map[g], m]
			nx.append((ki, g, np.mean(es), np.std(es)))
		cluster_expr[:, ki-1] = np.array([n[2] for n in nx])
		nt[ki] = pd.Series([tx[2] for tx in nx], index=nj_union_title)

	#print pd.DataFrame(nt)
	#f2,axn = plt.subplots(1,1, figsize=(9,20))
	#ax = sns.heatmap(pd.DataFrame(nt), annot=False, fmt=".2f", ax=axn)
	#sns.set(font_scale=1.0)
	cg=sns.clustermap(pd.DataFrame(nt), row_cluster=True, col_cluster=True, \
	figsize=(5, 20), method="average", vmax=vmax, vmin=vmin, col_ratios={"dendrogram":0.05}, \
	row_ratios={"dendrogram":0.05})
	plt.setp(cg.ax_heatmap.yaxis.get_majorticklabels(), rotation=0)
	if outfile is None:
		plt.show()
	else:
		cg.savefig(outfile)





import numpy as np
import math
import sys
import random
#import matplotlib.pyplot as plt

'''
def choose_random_cells(n, k):
	X = read_expression_matrix(n)
	ncell = X.shape[0]
	allcell = [x for x in range(ncell)]
	random.shuffle(allcell)
	ngene = X.shape[1]
	for i in range(k):
		sys.stdout.write("%d " % (i+1))
		for g in range(ngene):
			sys.stdout.write("%.5f" % X[allcell[i], g])
			if g==ngene-1:
				sys.stdout.write("\n")
			else:
				sys.stdout.write(" ")
'''	
def read_adjacency(n):
	f = open(n)
	m = {}
	for l in f:
		l = l.rstrip("\n").split(" ")
		pt = int(l[0])
		m.setdefault(pt, [])
		for j in l[1:]:
			if j=="-1":
				break
			m[pt].append(int(j))
	f.close()
	return m

def read_genes(n):
	f = open(n)
	genes = []
	for l in f:
		l = l.rstrip("\n")
		genes.append(l)
	f.close()
	return genes

def read_coordinates(n):
	ma, field = read_coord(n)
	ma = ma[1:, :]
	field = field[1:]
	return ma, field
	
def read_coord(n):
	f = open(n)
	m = {}
	field = 10
	for l in f:
		l = l.rstrip("\n").split(" ")
		pt = l[0]
		field = int(l[1])
		coord1 = float(l[2])
		coord2 = float(l[3])
		m[pt] = (field, coord1, coord2)
	f.close()
	ma = np.empty((len(m)+1, 2))
	ma[0] = [0, 0]
	field = np.arange(len(m)+1)
	field[0] = 0
	for k in m:
		ma[int(k)] = (m[k][1], m[k][2])
		field[int(k)] = m[k][0]
	return ma, field
	
#def read_classes(n, plot=False):
def read_classes(n):
	f = open(n)
	m = {}
	prob = []
	for l in f:
		l = l.rstrip("\n").split(" ")
		pt = l[0]
		cc = [float(e) for e in l[1:]]
		cl = np.argmax(cc) + 1
		prob.append(max(cc))
		m[pt] = cl
	f.close()
	ma = np.arange(len(m)+1)
	ma[0] = -1
	for k in m:
		ma[int(k)] = m[k]
	count0_05 = len([x for x in prob if 1-x>0.05])
	count0_01 = len([x for x in prob if 1-x>0.01])
	count0_2 = len([x for x in prob if 1-x>0.2])
	#count0.01 = 
	#count0.005 = 
	#count0.001 = 
	#print ["%.2e" % x for x in prob]
	#print count0_05, count0_01, count0_2

	#if plot:
	#	plt.hist(np.array(prob), bins=100, histtype="step")

	return ma
	
def read_cell_type(n):
	f = open(n)
	m = {}
	labels = set([])
	for l in f:
		l = l.rstrip("\n").split("\t")
		pt = l[0]
		m[int(pt)] = l[1]
		labels.add(l[1])
	f.close()

	dx = int(len(labels)/2)
	di = -1 * dx
	sort_labels = list(sorted(labels))

	label_pos = {}
	for i,s in enumerate(sort_labels):
		label_pos[s] = i
		
	s_labels = set(labels)
	print s_labels
	
	'''
	if "Glutamatergic Neuron" in s_labels and \
	"Endothelial Cell" in s_labels:
		tmp1 = label_pos["Glutamatergic Neuron"]
		tmp2 = label_pos["Endothelial Cell"]
		sort_labels[tmp1] = "Endothelial Cell"
		sort_labels[tmp2] = "Glutamatergic Neuron"
		label_pos["Glutamatergic Neuron"] = tmp2
		label_pos["Endothelial Cell"] = tmp1		
	
	if "Astrocyte" in s_labels and \
	"Endothelial Cell" in s_labels:
		tmp1 = label_pos["Astrocyte"]
		tmp2 = label_pos["Endothelial Cell"]
		sort_labels[tmp1] = "Endothelial Cell"
		sort_labels[tmp2] = "Astrocyte"
		label_pos["Astrocyte"] = tmp2
		label_pos["Endothelial Cell"] = tmp1	
	'''
	'''
	if "GABA-ergic Neuron" in s_labels and \
	"Glutamatergic Neuron" in s_labels and \
	"Oligodendrocyte" in s_labels:
		tmp1 = label_pos["GABA-ergic Neuron"]
		#tmp2 = label_pos["Glutamatergic Neuron"]
		tmp3 = label_pos["Oligodendrocyte"]
		sort_labels[tmp1] = "Oligodendrocyte"
		sort_labels[tmp3] = "GABA-ergic Neuron"
		#sort_labels[tmp2] = "Oligodendrocyte.3"
		#sort_labels[tmp4] = "Glutamatergic Neuron"
	'''
	dd = {}
	for s in sort_labels:
		if di==0:
			di+=1
		dd[s] = di
		di+=1
		
	ma = np.arange(len(m.keys())+1)
	ma[0] = -1
	for k in sorted(m.keys()):
		ma[k] = dd[m[k]]
	print sort_labels
	return ma, dd

def get_likelihood(n): #need unnormprob files
	'''
	w = {}
	tot_w = 0
	f = open(n)
	for l in f:
		l = l.rstrip("\n").split(" ")
		cc = [float(e) for e in l[1:]]
		cl = np.argmax(cc) + 1
		w.setdefault(cl, 0)
		w[cl]+=1
		tot_w+=1
	f.close()
	for wi in w:
		w[wi] = w[wi] / float(tot_w)
	'''
	
	f = open(n)
	prob = {}
	tot = 0
	for l in f:
		l = l.rstrip("\n").split(" ")
		cc = [float(e) for e in l[1:]]
		num_val = len(cc)
		tot_density = 0
		for i,j in zip(range(1,num_val+1), cc):
			tot_density += j
		this_prob = {}
		for i,j in zip(range(1,num_val+1), cc):
			this_prob[i] = j / tot_density
			prob.setdefault(i, 0)
			prob[i] += this_prob[i]
		tot+=1
	f.close()
	
	for i in prob:
		prob[i] = prob[i] / tot
		
	likelihood = 0
	f = open(n)
	for l in f:
		l = l.rstrip("\n").split(" ")
		cc = [float(e) for e in l[1:]]
		num_val = len(cc)
		this_like = 0
		for i,j in zip(range(1,num_val+1), cc):
			this_like += prob[i] * j
		likelihood+=math.log(this_like)
	f.close()
	likelihood /= tot
	return likelihood, tot
	
def read_clusters(n):
	f = open(n)
	m = {}
	for l in f:
		l = l.rstrip("\n").split(" ")
		pt = l[0]
		clust = int(l[1])
		m[pt] = clust
	f.close()
	ma = np.arange(len(m)+1)
	ma[0] = -1
	for k in m:
		ma[int(k)] = m[k]
	return ma
	
def read_kmeans(n):
	f = open(n)
	m = {}
	for l in f:
		l = l.rstrip("\n").split(" ")
		pt = l[0]
		cl = int(l[1])
		m[pt] = cl
	f.close()
	ma = np.arange(len(m)+1)
	ma[0] = -1
	for k in m:
		ma[int(k)] = m[k]
	return ma
	
def read_expression(n):
	f = open(n)
	m = {}
	for l in f:
		l = l.rstrip("\n").split(" ")
		pt = l[0]
		m[pt] = float(l[1])
	f.close()
	ma = np.arange(len(m)+1, dtype="float32")
	ma[0] = -1
	for k in m:
		ma[int(k)] = m[k]
	return ma

def read_expression_matrix(n, transpose=True):
	X = np.loadtxt(n, dtype="float32")
	X = X[:, 1:]
	if transpose:
		X = np.transpose(X)
	return X

def write_expression(x, fileName):
	fw = open(fileName, "w")
	for i in range(1, x.shape[0]):
		fw.write("%d %.5f\n" % (i, x[i]))
	fw.close()

def write_expression_matrix(x, fileName):
	fw = open(fileName, "w")
	for i in range(x.shape[0]):
		fw.write("%d " % (i+1))
		for j in range(x.shape[1]):
			fw.write("%.2f" % (x[i,j]))
			if j==x.shape[1]-1:
				fw.write("\n")
			else:
				fw.write(" ")
	fw.close()

def write_coordinate(Xcen, field, fileName):
	fw = open(fileName, "w")
	for i in range(1, field.shape[0]):
		fw.write("%d %d %.5f %.5f\n" % (i, field[i], Xcen[i,0], Xcen[i,1]))
	fw.close()
	



import sys
import math
import os
import numpy as np
import scipy
import scipy.stats
import random
from scipy.stats import zscore
from scipy.spatial.distance import euclidean,squareform,pdist
from abc import ABCMeta, abstractmethod
import subprocess
sys.setrecursionlimit(10000)

class DatasetMatrix:
	__metaclass__ = ABCMeta
	expr = None
	Xcen = None
	genes = []
	cells = []
	ngene = 0
	ncell = 0
	edges = None
	adjacent = None
	blocks = None

	def __init__(self, expr, genes, cells, Xcen):
		self.expr = expr
		self.genes = genes
		self.cells = cells
		self.Xcen = Xcen
		self.ngene = len(genes)
		self.ncell = Xcen.shape[0]
		self.edges = None
		self.adjacent = None
		self.blocks = None

	@abstractmethod
	def get_adjacency_list(self): pass

	@abstractmethod
	def test_adjacency_list(self): pass

	@abstractmethod
	def calc_neighbor_graph(self): pass

	@abstractmethod
	def subset_genes(self): pass 

	@abstractmethod
	def shuffle(self): pass

	def _get_shuffle_expression(self, shuffle_prop):
		new_expr = np.transpose(self.expr)
		row_order = np.empty((self.ncell), dtype="int32")
		for i in range(self.ncell):
			row_order[i] = i

		per = shuffle_prop
		#per = 0.99 #0.05, 0.10, 0.20, 0.50, 0.99
		shuf_cutoff = int(float(self.ncell) * per)
		row_order_copy = np.copy(row_order)
	
		ig = 0
		while True:
			this_sample = random.sample(range(0, self.ncell), 2)
			tmp = row_order_copy[this_sample[0]]
			row_order_copy[this_sample[0]] = row_order_copy[this_sample[1]]
			row_order_copy[this_sample[1]] = tmp
			num_changed = 0
			for i in range(self.ncell):
				if row_order_copy[i]!=row_order[i]:
					num_changed+=1
			if num_changed>shuf_cutoff:
				break
			ig +=1
	
		new_expr = new_expr[row_order_copy, :]
		new_expr = np.transpose(new_expr)
		return new_expr

	def write_neighbor_graph(self, adj_file=None, edge_file=None):
		print "Adjacency"
		maxNeighbor = max([len(self.adjacent[e]) for e in self.adjacent.keys()])
		fw = open(adj_file, "w")
		for i in range(self.ncell):
			numPad = maxNeighbor - len(self.adjacent[i])
			ix = [(e+1) for e in sorted(self.adjacent[i])]
			ix.extend([-1 for iv in range(numPad)])
			fw.write("%d " % (i+1) + " ".join(["%d" % e for e in ix]) + "\n")
		fw.close()
		print "Edges"
		fw = open(edge_file, "w")
		for e1, e2 in self.edges:
			fw.write("%d %d\n" % (e1+1, e2+1))
		fw.close()

	def calc_independent_region(self):
		edge_file = "/tmp/edges.txt"
		block_file = "/tmp/blocks.txt"
		adj_file = "/tmp/adjacent.txt"
		self.write_neighbor_graph(adj_file=adj_file, edge_file=edge_file)
		import smfishHmrf
		this_path = os.path.dirname(smfishHmrf.__file__) + "/graphColoring"
		subprocess.call("java -cp %s GraphColoring %s %s" % (this_path, edge_file, block_file), shell=True)
		#os.system("java -cp %s GraphColoring %s %s" % (this_path, edge_file, block_file))
		self.blocks = []
		f = open(block_file)
		for l in f:
			l = l.rstrip("\n")
			ll = l.split()
			self.blocks.append(int(ll[1]))
		f.close()
		self.blocks = np.array(self.blocks)

	def write_blocks(self, outfile):
		fw = open(outfile, "w")
		for i in range(self.blocks.shape[0]):
			fw.write("%d %d\n" % (i+1, self.blocks[i]))
		fw.close()

	def write_genes(self, outfile):
		fw = open(outfile, "w")
		for g in self.genes:
			fw.write(g + "\n")
		fw.close()
	
	def write_expression(self, outfile):
		fw = open(outfile, "w")
		for ic in range(self.ncell):	
			fw.write("%d " % (ic+1) + " ".join(["%.2f" % self.expr[j,ic] for j in range(self.ngene)]) + "\n")
		fw.close()
	
	def write_coordinates(self, outfile):
		fw = open(outfile, "w")
		for ic in range(self.ncell):
			fw.write("%d %d %.3f %.3f\n" % (ic+1, 0, self.Xcen[ic, 0], self.Xcen[ic, 1]))
		fw.close()

class DatasetMatrixMultiField(DatasetMatrix):
	cutoff = 0
	field = None

	def __init__(self, expr, genes, cells, Xcen, field):
		super(DatasetMatrixMultiField, self).__init__(expr, genes, cells, Xcen)
		self.cutoff = 0
		self.field = field

	def get_adjacency_list(self, cutoff=None):
		points = set([])
		edges = set([])
		adjacent = {}
		for i in range(self.ncell):
			for j in range(i+1, self.ncell):
				if self.field[i]!=self.field[j]: 
					continue
				this_dist = euclidean(self.Xcen[i,:], self.Xcen[j,:])
				if this_dist<=cutoff:
					edges.add(tuple(sorted([i, j])))
					points = points | set([i,j])
		for i in range(self.ncell):
			if i in points: continue
			dist_i = sorted([(euclidean(self.Xcen[i,:],self.Xcen[j,:]), j) \
			for j in range(self.ncell) if i!=j and self.field[i]==self.field[j]])
			edges.add(tuple(sorted([i, dist_i[0][1]])))
		for e1, e2 in edges:
			adjacent.setdefault(e1, set([]))
			adjacent.setdefault(e2, set([]))
			adjacent[e1].add(e2)
			adjacent[e2].add(e1)
		return edges, adjacent
	
	def test_adjacency_list(self, percentages, metric="euclidean"):
		output = []
		percentile = {}
		npair = 0
		for i in range(self.ncell):
			for j in range(i+1, self.ncell):
				if self.field[i]!=self.field[j]: continue
				npair+=1
		dist = np.empty((npair), dtype="float32")
		ix = 0
		for i in range(self.ncell):
			for j in range(i+1, self.ncell):
				if self.field[i]!=self.field[j]: continue
				dist[ix] = euclidean(self.Xcen[i,:], self.Xcen[j,:])
				ix+=1
		for px in percentages:
			percentile[px] = np.percentile(dist, px)
			edges, adjacent = self.get_adjacency_list(cutoff=percentile[px])
			avg_neighbor = np.mean([len(adjacent[n]) for n in adjacent])
			print "cutoff:%.2f%%" % px, "#nodes:%d" % len(adjacent), \
			"#edges:%d"%len(edges), "avg.nei:%.2f" % avg_neighbor
			output.append({"cutoff":px, "nodes":len(adjacent), "edges":len(edges), "avg.nei":avg_neighbor})
		return output

	def calc_neighbor_graph(self, cutoff, metric="euclidean"):
		npair = 0
		for i in range(self.ncell):
			for j in range(i+1, self.ncell):
				if self.field[i]==self.field[j]: 
					npair+=1
		dist = np.empty((npair), dtype="float32")
		ix = 0
		for i in range(self.ncell):
			for j in range(i+1, self.ncell):
				if self.field[i]!=self.field[j]: continue
				dist[ix] = euclidean(self.Xcen[i,:], self.Xcen[j,:])
				ix+=1
		percent_value = np.percentile(dist, self.cutoff)
		self.edges, self.adjacent = self.get_adjacency_list(cutoff=percent_value)
	
	def subset_genes(self, custom_genes):
		ind = np.array([self.genes.index(g) for g in custom_genes if g in set(self.genes)])
		new_expr = self.expr.copy()
		new_expr = new_expr[ind, :]
		new_dset = DatasetMatrixMultiField(new_expr, custom_genes, self.cells, self.Xcen, self.field)
		new_dset.edges = self.edges
		new_dset.adjacent = self.adjacent
		new_dset.blocks = self.blocks
		new_dset.cutoff = self.cutoff
		return new_dset

	def shuffle(self, shuffle_prop):
		new_expr = self._get_shuffle_expression(shuffle_prop)
		new_dset = DatasetMatrixMultiField(new_expr, self.genes, self.cells, self.Xcen, self.field)
		new_dset.edges = self.edges
		new_dset.adjacent = self.adjacent
		new_dset.blocks = self.blocks
		new_dset.cutoff = self.cutoff
		return new_dset		

#===========================================================


class DatasetMatrixSingleField(DatasetMatrix):
	cutoff = 0

	def __init__(self, expr, genes, cells, Xcen):
		super(DatasetMatrixSingleField, self).__init__(expr, genes, cells, Xcen)
		self.cutoff = 0

	def get_adjacency_list(self, dist=None, cutoff=None):
		ncell = dist.shape[0]
		points = set([])
		edges = set([])
		adjacent = {}
		for i in range(ncell):
			for j in range(i+1, ncell):
				if dist[i,j]<=cutoff:
					edges.add(tuple(sorted([i, j])))
					points = points | set([i,j])
		for i in range(ncell):
			if i in points: continue
			dist_i = sorted([(dist[i,j], j) for j in range(ncell) if i!=j])
			edges.add(tuple(sorted([i, dist_i[0][1]])))
		for e1, e2 in edges:
			adjacent.setdefault(e1, set([]))
			adjacent.setdefault(e2, set([]))
			adjacent[e1].add(e2)
			adjacent[e2].add(e1)
		return edges, adjacent
	
	def test_adjacency_list(self, percentages, metric="euclidean"):
		output = []
		percentile = {}
		dist = pdist(self.Xcen, metric=metric)
		s_dist = squareform(dist)
		for px in percentages:
			percentile[px] = np.percentile(dist, px)
			edges, adjacent = self.get_adjacency_list(dist=s_dist, cutoff=percentile[px])
			avg_neighbor = np.mean([len(adjacent[n]) for n in adjacent])
			print "cutoff:%.2f%%" % px, "#nodes:%d" % len(adjacent), \
			"#edges:%d"%len(edges), "avg.nei:%.2f" % avg_neighbor
			output.append({"cutoff":px, "nodes":len(adjacent), "edges":len(edges), "avg.nei":avg_neighbor})
		return output
	
	def calc_neighbor_graph(self, cutoff, metric="euclidean"):
		dist = pdist(self.Xcen, metric=metric)
		s_dist = squareform(dist)
		self.cutoff = cutoff
		percent_value = np.percentile(dist, self.cutoff)
		self.edges, self.adjacent = self.get_adjacency_list(dist=s_dist, cutoff=percent_value)
	
	def subset_genes(self, custom_genes):
		ind = np.array([self.genes.index(g) for g in custom_genes if g in set(self.genes)])
		new_expr = self.expr.copy()
		new_expr = new_expr[ind, :]
		new_dset = DatasetMatrixSingleField(new_expr, custom_genes, self.cells, self.Xcen)
		new_dset.edges = self.edges
		new_dset.adjacent = self.adjacent
		new_dset.blocks = self.blocks
		new_dset.cutoff = self.cutoff
		return new_dset

	def shuffle(self, shuffle_prop):
		new_expr = self._get_shuffle_expression(shuffle_prop)
		new_dset = DatasetMatrixSingleField(new_expr, self.genes, self.cells, self.Xcen)
		new_dset.edges = self.edges
		new_dset.adjacent = self.adjacent
		new_dset.blocks = self.blocks
		new_dset.cutoff = self.cutoff

		return new_dset		



import scipy
import scipy.stats
import sys
import re
import os
import numpy as np
import math
from scipy.spatial.distance import squareform, pdist
from scipy.stats import percentileofscore
import smfishHmrf.reader as reader

def read(n):
	f = open(n)
	h = f.readline().rstrip("\n").split(" ")[1:]
	num_g = len(h)
	num_c = 1
	for l in f:
		l = l.rstrip("\n")
		ll = l.split(" ")
		num_c+=1
	f.close()
	mat = np.empty((num_g, num_c), dtype="float32")

	f = open(n)
	c = 0
	for l in f:
		l = l.rstrip("\n")
		ll = l.split(" ")
		this_cell = ll[0]
		val = [float(v) for v in ll[1:]]
		mat[:,c] = val
		c+=1
	f.close()
	return mat

def get_distance_per_FD(mr_dissimilarity_FD, num_cell, clust):
	within_dist = {}
	across_dist = {}
	#m1 = np.where((field==100))[0]
	for i in range(num_cell):
		this_i = i
		within_dist.setdefault(this_i, [])
		across_dist.setdefault(this_i, {})
		for j in range(i+1, num_cell):
			this_j = j
			within_dist.setdefault(this_j, [])
			across_dist.setdefault(this_j, {})
			dist = mr_dissimilarity_FD[i, j]
			if clust[this_i]==clust[this_j]:
				within_dist[this_i].append(dist)
				within_dist[this_j].append(dist)
			else:
				across_dist[this_i].setdefault(clust[this_j], [])
				across_dist[this_j].setdefault(clust[this_i], [])
				across_dist[this_i][clust[this_j]].append(dist)
				across_dist[this_j][clust[this_i]].append(dist)
	silhouette = {}
	np_silhouette = np.zeros((num_cell), dtype="float32")
	#k = 10
	for pt in within_dist:
		if len(within_dist[pt])<5:
			continue
		mm = np.mean(within_dist[pt])
		across = []
		for cl2 in across_dist[pt]:
			if len(across_dist[pt][cl2])<5:
				continue
			#across_sort = np.sort(across_dist[pt][cl2])
			across.append(np.mean(across_dist[pt][cl2]))
			#across.append(np.mean(across_sort[:k]))
		if len(across)==0:
			continue
		mn = np.min(across)
		silhouette[pt] = float(mn - mm)/max(mn, mm)
		np_silhouette[pt] = silhouette[pt]
	avg_silhouette = 0
	for pt in silhouette:
		avg_silhouette+=silhouette[pt]
	avg_silhouette/=len(silhouette.keys())
	return avg_silhouette, np_silhouette	

def rank_transform_matrix(mat, rbp_p = 0.99, reverse=True):
	dim1 = mat.shape[0]
	dim2 = mat.shape[1]
	rank_forward = np.empty([dim1, dim2])
	for c1 in range(dim1):
		rd = scipy.stats.rankdata(mat[c1,:])
		if reverse==True:
			rd = dim2 - rd + 1
		rank_forward[c1, :] = rd
	rank_backward = np.empty([dim1, dim2])
	for c1 in range(dim2):
		rd = scipy.stats.rankdata(mat[:,c1])
		if reverse==True:
			rd = dim1 - rd + 1
		rank_backward[:, c1] = rd
	mutual_rank_rbp = np.empty([dim1, dim2])
	mutual_rank = np.empty([dim1, dim2])
	for c1 in range(dim1):
		for c2 in range(dim2):
			ma = math.sqrt(rank_forward[c1, c2] * rank_backward[c1, c2])
			mutual_rank_rbp[c1, c2] = (1-rbp_p) * math.pow(rbp_p, ma - 1) 
			mutual_rank[c1, c2] = ma
	dissimilarity = np.empty([dim1, dim2])
	for c1 in range(dim1):
		for c2 in range(dim2):
			dissimilarity[c1, c2] = 1 - mutual_rank_rbp[c1, c2] / (1-rbp_p)
	return dissimilarity

def calc_silhouette_per_gene(genes=None, expr=None, dissim=None, examine_top=0.1, permutation_test=True, permutations=100):
	if genes is None or expr is None or dissim is None:
		sys.stderr.write("Need genes, expr, dissim\n")
		return ;

	print "Started"
	sil = []
	ncell = expr.shape[1]

	ex = int((1.0-examine_top)*100.0)
	for ig,g in enumerate(genes):
		print g, ig, "/", len(genes)
		cutoff = np.percentile(expr[ig,:], ex)
		clust = np.zeros((ncell), dtype="int32")
		clust[np.where(expr[ig,:]>=cutoff)[0]] = 1
		clust[np.where(expr[ig,:]<cutoff)[0]] = 2
		avg_sil_rank, all_silhouette = get_distance_per_FD(dissim, ncell, clust)
		sil.append((g, avg_sil_rank, np.mean(all_silhouette[np.where(clust==1)[0]])))

	res = []
	if permutation_test:
		print "Permutation test..."
		rand_clust = np.zeros((ncell), dtype="int32")
		rand_clust[0:int(ncell*(1.0 - examine_top))] = 2
		rand_clust[int(ncell*(1.0 - examine_top)):] = 1
		rand_sil = []
		for i in range(permutations):
			if i%10==0:
				print "Done", i, "/", permutations
			np.random.shuffle(rand_clust)
			r_avg_sil_rank, r_all_silhouette = get_distance_per_FD(dissim, ncell, rand_clust)
			rand_sil.append((r_avg_sil_rank, np.mean(r_all_silhouette[np.where(rand_clust==1)[0]])))

		r_avg = np.array([a[0] for a in rand_sil])
		r_sil = np.array([a[1] for a in rand_sil])
		for ig,g in enumerate(genes):
			this_avg = sil[ig][1]
			this_sil = sil[ig][2]
			#print g, this_avg, this_sil, scipy.stats.percentileofscore(r_avg, this_avg), \
			#scipy.stats.percentileofscore(r_sil, this_sil)
			res.append((g, this_sil, (100.0 - percentileofscore(r_sil, this_sil))/100.0))
	else:
		for ig,g in enumerate(genes):
			res.append((g, this_sil))
	res.sort(lambda x,y:cmp(x[1], y[1]), reverse=True)
	return res


if __name__=="__main__":
	#cortex_FDs = [100]
	directory = "hmrfem.whole.brain.e.fd0-48.correlated.gene"
	roi = "cortex"
	#FDs = cortex_FDs
	workdir = "/home/qz64/hmrf.submit"
	genes = reader.read_genes("%s/%s/genes" % (workdir, directory))
	Xcen, field = reader.read_coord("%s/%s/pca_corrected_cortex/f%s.coordinates.txt" % (workdir, directory, roi)) #note the stitched version use the .2 version
	Xcen = Xcen[1:]
	field = field[1:]
	#mat = read("%s/%s/bias_corrected_cortex/f%s.expression.txt" % (workdir, directory, roi))
	mat = np.transpose(reader.read_expression_matrix("%s/%s/pca_corrected_cortex/f%s.expression.txt" % (workdir, directory, roi)))
	#print expr.shape

	print Xcen.shape
	ncell = Xcen.shape[0]
	print "Euclidean distance"
	euc = squareform(pdist(Xcen, metric="euclidean"))
	print "Rank transform"
	dissim = rank_transform_matrix(euc, reverse=False, rbp_p=0.95)
	
	res = calc_silhouette_per_gene(genes=genes, expr=mat, dissim=dissim, examine_top=0.1, permutation_test=True, permutations=100)
	for i,j,k in res:
		print i,j,k



import re
import sys
import os
import numpy as np
import math
from scipy.spatial.distance import squareform, pdist
import smfishHmrf.reader as reader
from smfishHmrf.DatasetMatrix import DatasetMatrix, DatasetMatrixSingleField
import subprocess

class HMRFInstance:
	dest = ""
	prefix = ""
	dataset = None #must be DatasetMatrix
	beta_incr = 0
	start_beta = 0
	num_beta = 0
	tolerance = 1e-60
	k = []
	nstart = 1000
	likelihood = {}
	domain = {}

	def prepare(self):
		self.dataset.write_neighbor_graph(adj_file="%s/f%s.adjacency.txt" % (self.dest, self.prefix), \
		edge_file="%s/f%s.edges.txt" % (self.dest, self.prefix))
		self.dataset.write_coordinates("%s/f%s.coordinates.txt" % (self.dest, self.prefix))
		self.dataset.write_blocks("%s/f%s.blocks.txt" % (self.dest, self.prefix))
		self.dataset.write_genes("%s/genes" % self.dest)
		self.dataset.write_expression("%s/f%s.expression.txt" % (self.dest, self.prefix))

		fw = open("%s/generateCentroids.multi.R" % self.dest, "w")
		fw.write("""par_k <- commandArgs(trailingOnly = TRUE)[1]
nstart <- commandArgs(trailingOnly = TRUE)[2]
mem_file <- commandArgs(trailingOnly = TRUE)[3]
centroid_file <- commandArgs(trailingOnly = TRUE)[4]
kmeans_file <- commandArgs(trailingOnly = TRUE)[5]

par_k <- as.integer(par_k)
nstart <- as.integer(nstart)
y<-read.table(mem_file, header=F, row.names=1)
y<-as.matrix(y)
k<-par_k
m<-dim(y)[2]
kk<-kmeans(y, k, nstart=nstart, iter.max=100)

write.table(kk$cluster, file=kmeans_file, sep=" ", quote=F, col.names=F, row.names=T)
write.table(kk$centers, file=centroid_file, sep=" ", quote=F, col.names=F, row.names=T)
		""")
		fw.close()

		fw = open("%s/getHMRFEM.multi.beta.auto.R" % self.dest, "w")
		fw.write("""library(dplyr)
library(smfishHmrf)
print("Loaded smfishHmrf")
mem_file <- commandArgs(trailingOnly = TRUE)[1]
nei_file <- commandArgs(trailingOnly = TRUE)[2]
block_file <- commandArgs(trailingOnly = TRUE)[3]
centroid_file <- commandArgs(trailingOnly = TRUE)[4]
beta <- commandArgs(trailingOnly = TRUE)[5]
beta_increment <- commandArgs(trailingOnly = TRUE)[6]
beta_num_iter <- commandArgs(trailingOnly = TRUE)[7]
par_k <- commandArgs(trailingOnly = TRUE)[8]
outdir <- commandArgs(trailingOnly = TRUE)[9]
prefix <- commandArgs(trailingOnly = TRUE)[10]
tolerance <- commandArgs(trailingOnly = TRUE)[11]

beta<-as.double(beta)
beta_increment<-as.double(beta_increment)
beta_num_iter<-as.integer(beta_num_iter)
par_k <- as.integer(par_k)
tolerance <- as.double(tolerance)

#file reading
y<-read.table(mem_file, header=F, row.names=1)
y<-as.matrix(y)
nei<-read.table(nei_file, header=F, row.names=1)
colnames(nei)<-NULL
rownames(nei)<-NULL
nei<-as.matrix(nei)
blocks<-read.table(block_file, header=F, row.names=1)
blocks<-c(t(blocks))
maxblock <- max(blocks)
blocks<-lapply(1:maxblock, function(x) which(blocks == x))
numnei<-apply(nei, 1, function(x) sum(x!=-1))
centroid<-read.table(centroid_file, header=F, row.names=1)
centroid<-as.matrix(centroid)
#parameter setting
k<-par_k
m<-dim(y)[2]
sigma <-array(0, c(m,m,k))
for(i in 1:k){
	sigma[, ,i] <- cov(y)
	print(rcond(sigma[,,i]))
}
mu<-array(0, c(m,k))
kk2<-centroid
for(i in 1:k){
	mu[,i] <- kk2[i,]
}
numcell<-dim(y)[1]
kk_dist<-array(0, c(numcell, k))
for(i in 1:numcell){
	for(j in 1:k){
		kk_dist[i,j] <- dist(rbind(y[i,], mu[,j]), method="euclidean")
	}
}
clust_mem<-apply(kk_dist, 1, function(x) which(x==min(x))) 
lclust<-lapply(1:k, function(x) which(clust_mem == x))
damp<-array(0, c(k))
for(i in 1:k){
	sigma[, , i] <- cov(y[lclust[[i]], ])
	#default tolerance is 1e-60
	di<-findDampFactor(sigma[,,i], factor=1.05, d_cutoff=tolerance, startValue=0.0001)
    if(is.null(di)){
        damp[i] = 0
    }else{
        damp[i] = di
    }
}

#needs y, nei, beta, numnei, blocks, mu, sigma, damp
do_one <- function(prefix, outdir, #all strings
                   par_k, par_y, par_nei, par_beta, par_numnei, par_blocks, 
                   par_mu, par_sigma, par_damp){ #beta is double, par_k is integer

out_file <- sprintf("%s/%s.%.1f.prob.txt", outdir, prefix, par_beta) #hmrfem probability
out_file_2 <- sprintf("%s/%s.%.1f.centroid.txt", outdir, prefix, par_beta) #hmrfem centroids
out_file_3 <- sprintf("%s/%s.%.1f.hmrf.covariance.txt", outdir, prefix, par_beta) #hmrfem covariance
out_file_unnorm <- gsub("prob", "unnormprob", out_file)

tc.hmrfem<-smfishHmrf.hmrfem.multi(y=par_y, neighbors=par_nei, beta=par_beta, numnei=par_numnei, 
blocks=par_blocks, mu=par_mu, sigma=par_sigma, verbose=T, 
err=1e-7, maxit=50, dampFactor=par_damp)

write.table(tc.hmrfem$prob, file=out_file, sep=" ", quote=F, col.names=F, row.names=T)
write.table(tc.hmrfem$unnormprob, file=out_file_unnorm, sep=" ", quote=F, col.names=F, row.names=T)
write.table(t(tc.hmrfem$mu), file=out_file_2, sep=" ", quote=F, col.names=F, row.names=T)
write.table(tc.hmrfem$sigma[,,1], file=out_file_3, sep=" ", quote=F, col.names=F, row.names=T)
for(i in 2:par_k){
	write.table(tc.hmrfem$sigma[,,i], file=out_file_3, sep=" ", quote=F, col.names=F, row.names=T, append=T)
}
}

beta_current <- beta
for(bx in 1:beta_num_iter){
	print(sprintf("%.3f", beta_current))
	do_one(prefix, outdir, k, y, nei, beta_current, numnei, blocks, mu, sigma, damp)
	beta_current <- beta_current + beta_increment
}
	""")
		fw.close()


		#os.system("chmod a+x %s/generateCentroids.multi.R" % self.dest)
		#os.system("chmod a+x %s/getHMRFEM.multi.beta.auto.R" % self.dest)

	#def __init__(self, prefix, dest, dataset, k, start_beta, beta_incr, num_beta, tolerance=1e-60):
	def __init__(self, prefix, dest, dataset, *args, **kwargs):

		use_existing_init = kwargs.get("use_existing_unit", False)
		tolerance = kwargs.get("tolerance", 1e-60)

		assert isinstance(dataset, DatasetMatrix), "dataset is not of DatasetMatrix"
		assert len(args)==4 or len(args)==2, "Number of arguments incorrect. Should be k, followed by beta. See manual"

		if len(args)==4:	#k, start_beta, beta_incr, num_beta (legacy reason)
			self.k = [args[0],]
			self.start_beta = args[1]
			self.beta_incr = args[2]
			self.num_beta = args[3]
		else:
			k = args[0]
			beta = args[1]
			assert (isinstance(beta, (list, tuple)) and len(beta)==3) or isinstance(beta, float), \
			"requires beta to be specified as either (start_beta, beta_incr, num_beta) or a single beta_val"
			assert (isinstance(k, (list, tuple)) and len(k)==3) or isinstance(k, (int, long)), \
			"requires k to be specified as either (start_k, k_incr, num_k) or a single k_val"
	
			if isinstance(beta, (list, tuple)):
				self.start_beta = beta[0]
				self.beta_incr = beta[1]
				self.num_beta = beta[2]
			else: #len(args)==1
				self.start_beta = beta
				self.beta_incr = 1.0
				self.num_beta = 1

			if isinstance(k, (list, tuple)):
				self.k = []
				#(start_k, k_incr, num_k)
				for t_k in range(k[2]):
					self.k.append(k[0] + k[1]*t_k)
			else:
				self.k = [k,]

		if use_existing_init:
			missing = False
			for t_k in self.k:
				o_dir = "k_%d" % t_k
				centroid_file = "%s/f%s.gene.ALL.centroid.txt" % (o_dir, prefix)
				kmeans_file = "%s/f%s.gene.ALL.kmeans.txt" % (o_dir, prefix)
				if os.path.isfile(centroid_file) and os.path.isfile(kmeans_file):
					sys.stderr.write("Centroid and Kmeans files for k=%d are missing" % t_k)
					missing = True
					break
			assert missing, "Use_existing_init=True, but some centroid and kmeans are missing..."

		self.dest = dest
		self.tolerance = tolerance
		self.dataset = dataset
		self.prefix = prefix
		self.nstart = 1000
		if kwargs.get("no_prepare", False)==False:
			self.prepare()
		self.likelihood = {}
		self.domain = {}

	@classmethod
	def ReadFrom(cls, prefix, dest, dataset, k, beta):
		hm = HMRFInstance(prefix, dest, dataset, k, beta, no_prepare=True)
		cur = os.getcwd()
		os.chdir(hm.dest)
		for t_k in hm.k:
			o_dir = "k_%d" % t_k
			for t_b in range(hm.num_beta):
				this_beta = hm.start_beta + t_b * hm.beta_incr
				lik, tot = reader.get_likelihood("%s/f%s.beta.%.1f.unnormprob.txt" % (o_dir, hm.prefix, this_beta))
				hm.likelihood[(t_k, this_beta)] = lik * tot
				hm.domain[(t_k, this_beta)] = reader.read_classes("%s/f%s.beta.%.1f.prob.txt" % (o_dir, hm.prefix, this_beta))[1:]

		os.chdir(cur)
		return hm

	def change_k(self, k):
		assert (isinstance(k, (list, tuple)) and len(k)==3) or isinstance(k, (int, long)), \
		"requires k to be specified as either (start_k, k_incr, num_k) or a single k_val"
		if isinstance(k, (list, tuple)):
			self.k = []
			#(start_k, k_incr, num_k)
			for t_k in range(k[2]):
				self.k.append(k[0] + k[1]*t_k)
		else:
			self.k = [k,]
		
	def change_beta(self, beta):
		assert (isinstance(beta, (list, tuple)) and len(beta)==3) or isinstance(beta, float), \
		"requires beta to be specified as either (start_beta, beta_incr, num_beta) or a single beta_val"
		if isinstance(beta, (list, tuple)):
			self.start_beta = beta[0]
			self.beta_incr = beta[1]
			self.num_beta = beta[2]
		else: #len(args)==1
			self.start_beta = beta
			self.beta_incr = 1.0
			self.num_beta = 1

	def change_tolerance(self, tolerance):
		self.tolerance = tolerance

	def init(self, nstart=1000):
		self.nstart = nstart
		cur = os.getcwd()
		os.chdir(self.dest)
		#input file (need to generate)
		expr_file = "f%s.expression.txt" % self.prefix

		for t_k in self.k:
			o_dir = "k_%d" % t_k
			if not os.path.isdir(o_dir):
				os.mkdir(o_dir)
			#output files
			centroid_file = "%s/f%s.gene.ALL.centroid.txt" % (o_dir, self.prefix)
			kmeans_file = "%s/f%s.gene.ALL.kmeans.txt" % (o_dir, self.prefix)
			#os.system("Rscript generateCentroids.multi.R %d %d %s %s %s" % (t_k, self.nstart, expr_file, centroid_file, \
			#kmeans_file))
			subprocess.call("Rscript generateCentroids.multi.R %d %d %s %s %s" % (t_k, \
			self.nstart, expr_file, centroid_file, kmeans_file), shell=True)

		os.chdir(cur)

	def run(self):
		cur = os.getcwd()
		os.chdir(self.dest)
		adjacency_file = "f%s.adjacency.txt" % self.prefix
		block_file = "f%s.blocks.txt" % self.prefix
		expr_file = "f%s.expression.txt" % self.prefix
		for t_k in self.k:
			o_dir = "k_%d" % t_k
			centroid_file = "%s/f%s.gene.ALL.centroid.txt" % (o_dir, self.prefix)
			o_file_prefix = "f%s.beta" % self.prefix
			cmd = "Rscript getHMRFEM.multi.beta.auto.R %s %s %s %s %.1f %.1f %d %d %s %s %.1e" % (\
			expr_file, adjacency_file, block_file, centroid_file, self.start_beta, \
			self.beta_incr, self.num_beta, t_k, o_dir, o_file_prefix, self.tolerance)
			#os.system(cmd)
			subprocess.call(cmd, shell=True)

		for t_k in self.k:
			o_dir = "k_%d" % t_k
			for t_b in range(self.num_beta):
				this_beta = self.start_beta + t_b * self.beta_incr
				lik, tot = reader.get_likelihood("%s/f%s.beta.%.1f.unnormprob.txt" % (o_dir, self.prefix, this_beta))
				self.likelihood[(t_k, this_beta)] = lik * tot
				self.domain[(t_k, this_beta)] = reader.read_classes("%s/f%s.beta.%.1f.prob.txt" % (o_dir, self.prefix, this_beta))[1:]

		os.chdir(cur)
	
	def plot_domain(self, this_k, this_beta, dot_size=45, lim=5000, size_factor=10, out_file=None):
		f, axn = plt.subplots(1, 1, figsize=(size_factor, size_factor), sharex=True, sharey=True)
		plt.subplots_adjust(hspace=0.01, wspace=0.01)
		cm = plt.cm.get_cmap("RdYlBu_r")
		cls = self.domain[(this_k, this_beta)]
		axn.scatter(self.Xcen[:,0], self.Xcen[:,1], s=dot_size, c=cls, edgecolors="gray", cmap=cm, vmin=1, vmax=max(cls))
		axn.set_xlim(0, lim)
		axn.set_ylim(-1 * lim, 0)
		f.savefig(out_file)
		



import subprocess
from distutils.command.build import build as _build
import setuptools


with open("README.md", "r") as fh:
	long_description = fh.read()


# This class handles the pip install mechanism.
class build(_build):  # pylint: disable=invalid-name
  sub_commands = _build.sub_commands + [("CustomCommands", None)]

CUSTOM_COMMANDS = [
	["libdir=`ls -1 build|grep \"lib\"`; cd build/$libdir/smfishHmrf/graphColoring && javac GraphColoring.java"]]

class CustomCommands(setuptools.Command):
  """A setuptools Command class able to run arbitrary commands."""

  def initialize_options(self):
    pass

  def finalize_options(self):
    pass

  def RunCustomCommand(self, command_list):
    print "Running command: %s" % command_list
    p = subprocess.Popen(
        command_list,
        stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)
    # Can use communicate(input='y\n'.encode()) if the command run requires
    # some confirmation.
    stdout_data, _ = p.communicate()
    print "Command output: %s" % stdout_data
    if p.returncode != 0:
      raise RuntimeError(
          "Command %s failed: exit code: %s" % (command_list, p.returncode))

  def run(self):
    for command in CUSTOM_COMMANDS:
      self.RunCustomCommand(command)



setuptools.setup(
	name="smfishHmrf",
	version="1.0.8.1",
	author="Qian Zhu",
	author_email="zqian@jimmy.harvard.edu",
	description="HMRF for spatial transcriptomic data",
	long_description="This tool infers spatial domain-like patterns from spatial gene expression data that are generated by multiplexed single molecule FISH or by other spatial profiling technologies",
	long_description_content_type="text/markdown",
	url="https://bitbucket.org/qzhu/smfish-hmrf",
	packages=setuptools.find_packages(),
	classifiers=(
		"Programming Language :: Python :: 2.7",
		"License :: OSI Approved :: MIT License",
		"Operating System :: OS Independent",
	),
	package_data={"smfishHmrf":  ["graphColoring/Backtracking.java", 
		"graphColoring/Clique.java", "graphColoring/Constants.java", 
		"graphColoring/IteratedGreedy.java", "graphColoring/LocalSearch.java", 
		"graphColoring/MaxClique.java", "graphColoring/Node.java",
		"graphColoring/SortedListNode.java", "graphColoring/Graph.java", 
		"graphColoring/GraphReader.java", "graphColoring/GraphColoring.java", 
		"graphColoring/Clique.class", 
		"graphColoring/Constants.class",
		"graphColoring/Graph.class",
		"graphColoring/GraphColoring.class",
		"graphColoring/GraphColoring$PossibleColorsComparator.class",
		"graphColoring/GraphReader.class",
		"graphColoring/IteratedGreedy.class",
		"graphColoring/IteratedGreedy$ColorClass.class",
		"graphColoring/IteratedGreedy$DecreasingComparator.class",
		"graphColoring/IteratedGreedy$IncreasingComparator.class",
		"graphColoring/LocalSearch.class",
		"graphColoring/MaxClique.class",
		"graphColoring/Node.class",
		"graphColoring/SortedListNode.class"]},
	install_requires=[
		"scipy", "numpy", "pandas", "seaborn", "matplotlib"],
	cmdclass={
		"build": build,
		"CustomCommands": CustomCommands,
		}	
)
	
